/* $Id: getnameinfo.c,v 1.1.1.1 1996/10/31 12:51:32 pan Exp $

   Implementation of the getnameinfo function as defined by IEEE P1003.1g
   document.


   Implementation of getnexthost for analysis of this file. This function
   returns a pointer to a struct hostent
 */

#include <sys/param.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <arpa/nameser.h>

#include <stdio.h>
#include <netdb.h>
#include <resolv.h>
#include <ctype.h>
#include <errno.h>
#include <syslog.h>
#include <string.h>

#define MAXALIASES 		35



/* Gets the next line of /etc/hosts and returns it in a hostent structure
   either if the address is AF_INET or AF_INET6
 */
struct hostent *getnexthost(int close)
{
	static struct hostent host;	/* for the result */
	static FILE *hostf = NULL;	/* the etc/hosts file */
	char *p, *cp, *host_n;
	static char addr[16], *h_ad[2];
	static char *h_alias[MAXALIASES + 1];
	static char buf[BUFSIZ];
	int res, nb_names;;

	if (close > 0) {	/* for file closing */
		fclose(hostf);
		hostf = NULL;
		return (NULL);
	}
	h_ad[0] = NULL;
	h_ad[1] = NULL;
	host.h_addr_list = h_ad;
	if ((hostf == NULL) && ((hostf = fopen("/etc/hosts", "r")) == NULL)) {
		return (NULL);
	}
	while ((p = fgets(buf, sizeof(buf), hostf)) != NULL) {
		if (*p == '#') {
			continue;	/* a comment */
		}
		if ((cp = strpbrk(p, "#\n")) == NULL) {
			continue;	/* no end of line !!! */
		}
		*cp = '\0';
		while (isspace(*p))
			p++;
		if ((cp = strpbrk(p, " \t")) == NULL) {
			continue;	/* no field separation */
		}
		*cp = '\0';
		cp++;
		
		host_n = NULL;	/* beginning of host name */
		/* trying AF_INET address */
		if (inet_pton(AF_INET, p, addr))  {	
			/* this is an IPv4 address */
			host.h_addrtype = AF_INET;
			host.h_length = INADDRSZ;
			host.h_addr = addr;
			host_n = cp;
		}	
		else {	/* if ... IPv4 address */
		   /* trying AF_INET6 address */
		   if (inet_pton(AF_INET6, p, addr)) {
			/* this is an IPv6 address */
			host.h_addrtype = AF_INET6;
			host.h_length = IN6ADDRSZ;
			host.h_addr = addr;
			host_n = cp;
		   }		/* if ... IPv6 address */
		}
		
		if (host_n != NULL) {	/* analysis of names */
			p = host_n;
			nb_names = 0;
			while (*host_n != '\0') {
				if (isspace(*host_n)) {
					if (p != host_n) {	/* end of a name */
						*host_n = '\0';
						host_n++;
						if (nb_names < MAXALIASES) {
							h_alias[nb_names] = p;
							nb_names++;
						}
						p = host_n;
					} else {
						host_n++;
						p++;
					}	/* p != host_n */
				} else {
					host_n++;
				}	/* if isspace .. */
			}	/* while */
			if (p != host_n) {	/* last name of the list */
				h_alias[nb_names] = p;
				nb_names++;
			}
			h_alias[nb_names] = NULL;
			host.h_name = h_alias[0];
			host.h_aliases = h_alias;
			return (&host);
		}		/* end of name analysis */
	}			/* while */
	return (NULL);
}				/* getnexthost */



int getnameinfo(const struct sockaddr *sa, size_t addrlen,
		char *host, size_t hostlen, char *serv, size_t servlen)
{
	register struct servent *p;
	register struct hostent *q;
	struct sockaddr_in *sin;
	struct sockaddr_in6 *sin6;
	int res = -1;


	if (servlen > 0) {	/* service name requested */
		while (p = getservent()) {
			if (p->s_port == ((struct sockaddr_in6 *) sa)->sin6_port) {
				strncpy(serv, p->s_name, servlen);
				res = 0;
				break;
			}
		}		/* while */
		endservent();	/* /etc/services is closed */
		if (p == NULL) {
			h_errno = HOST_NOT_FOUND;	/* to be modified */
			return (NETDB_INTERNAL);
		}
	}			/* if ... service name requested */
	
	
	if (hostlen > 0) {	/* machine name requested */
 	/* make a request for the name server */
 	
 		switch (sa->sa_family){
 		case AF_INET:
 			sin = (struct sockaddr_in *) sa;
 			q = gethostbyaddr ( (char *) &sin->sin_addr, INADDRSZ, AF_INET);
 		case AF_INET6:
 			sin6 = (struct sockaddr_in6 *) sa;
 			q = gethostbyaddr ( (char *) &sin6->sin6_addr, IN6ADDRSZ, AF_INET6);
 		default:
			errno = EAFNOSUPPORT;
			return (NETDB_INTERNAL);
 		} /* switch */
 	 	if (!q) {
 	 		h_errno = HOST_NOT_FOUND;	/* to be modified */
			return (NETDB_INTERNAL);
		}
 		strncpy(host, q->h_name, hostlen);

	}			/* if ... machine name requested */
	return (res);
}   /* getnameinfo */

